# Name:        findmodelelementswithcode.pl
# Author:      Anders Olsson (aolsson@rational.com)
# Description: Find model elements that has a code block. 
#              Optionally a regular expression limits the result.
# Usage: rtperl findmodelelementswithcode.pl INFILES OUTPUTFILE REGEXP
#            INFILES    - input file which specifies all model files to search
#            OUTPUTFILE - the output file which lists all resulting object quid:s
#            REGEXP     - if the code block matches this regular expresion,
#                         it will be included in the result, otherwise not

my($infiles, $output, $regexp) = @ARGV;
open(INFILES, $infiles) || die "Can't open $infiles for reading: $!";
my $olddelimiter = $/;
my $outputstr = "";

# for every model file
while (<INFILES>){
   open(IN, $_) || die "Can't open $_ for reading: $!";
   # read in the whole file
   undef $/;
   while (<IN>) {
      while ( /
                ^\s*quid\s"(.*)".*\n    # first the quid ...
                (?:                     # ... then two possible situations
                   ^\s*body\s"(.*)".*\n # 1: body "single-line"
                |
                   ^\s*body\s*\n        # 2: multi-line body
	             ^(?:\|.*\n)+
                )
              /xmg ) {
         my $quid = $1;
         if ($regexp) {
            my $body = $2; # single-line body is caught in the regexp
            if (!$body){   # multi-line situation is a bit more complicated...
               $body = $&;
               $body =~ s/(^.*quid\s".*"\s*\n^.*body\s*\n|^\|)//mg; 
            }
            if ($body =~ /$regexp/mg){ # use the regexp parameter
               $outputstr = "$outputstr$quid\n";
            }
         } else {
            $outputstr = "$outputstr$quid\n";
         }
      }
   }
   $/ = $olddelimiter;
}
open(OUT, ">$output") || die "Can't open $output for writing: $!";
print OUT $outputstr;



